#include <ClanLib/core.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/sdl.h>

class JoystickApp : public CL_ClanApplication
{
	void on_joystick_down(const CL_InputEvent &key)
	{
		std::cout << CL_Joystick::get_key_name(key.id) << std::endl;
	}

	void on_joystick_move(const CL_InputEvent &key)
	{
		std::cout << key.mouse_pos.x << "," << key.mouse_pos.y << std::endl;
	}

public:
	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();

		try
		{
			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupSDL::init();

			CL_DisplayWindow window("Joystick Example", 640, 480);
			
			CL_Slot slot_joystick_move = CL_Joystick::sig_move().connect(this, &JoystickApp::on_joystick_move);
			CL_Slot slot_joystick_button = CL_Joystick::sig_key_down().connect(this, &JoystickApp::on_joystick_down);
		
			// Loop until the user presses Escape
			while (CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == false)
			{
				// Clear display
				CL_Display::clear(CL_Color(0, 0, 128));
			
				// Flip front and backbuffer - this makes the changes visible
				CL_Display::flip();
			
				CL_System::sleep(10);

				// Update keyboard input and handle system events
				CL_System::keep_alive();
			}
		}
		catch (CL_Error err)
		{
			std::cout << "Exception caught: " << err.message.c_str() << std::endl;
		}

		// Display console close message and wait for a key
		console.display_close_message();

		CL_SetupSDL::deinit();
		CL_SetupDisplay::deinit();
		CL_SetupCore::deinit();

		return 0;
	}
} app;
