/*  $Id: progressbar_generic.cpp,v 1.9 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "progressbar_generic.h"
#include "API/GUI/progressbar.h"
#include "API/GUI/component.h"
#include "API/GUI/component_options.h"
#include "API/GUI/stylemanager.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_ProgressBar_Generic::CL_ProgressBar_Generic(
	CL_ProgressBar *self,
	int steps)
:
	progress(0),
	progressbar(self)
{
	set_steps(steps);

	slot_set_options = progressbar->sig_set_options().connect(
		this, &CL_ProgressBar_Generic::on_set_options);
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

float CL_ProgressBar_Generic::get_percentage() const
{
	return (float)progress / steps;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_ProgressBar_Generic::set_steps(int total_steps)
{
	steps = total_steps;

	if(progress > steps)
		progress = steps;
}

void CL_ProgressBar_Generic::set_progress(int new_progress)
{
	progress = new_progress;
	if(progress > steps)
		progress = steps;
	if(progress < 0)
		progress = 0;
}

void CL_ProgressBar_Generic::increase(int steps)
{
	set_progress(progress + steps);
}

void CL_ProgressBar_Generic::reset()
{
	progress = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_ProgressBar_Generic::on_set_options(const CL_ComponentOptions &options)
{
	if (options.exists("steps"))
		set_steps(options.get_value_as_int("steps"));
}
